﻿Imports System.Xml

Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.Serialize
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles
Imports ThinkGeo.MapSuite.WinForms

Public Class frmMain
  Private WithEvents moWmtsLayer As New WmtsLayer()
  Private sDataPath As String = "..\..\Data\"
  Private moLayer As DataTable
  Private moLayerStyle As DataTable
  Private moTableMatrixLink As DataTable
  Private moWGS84BoundingBox As DataTable
  Private Sub Form1_Load(sender As Object, e As EventArgs) Handles Me.Load
    Dim oProj4 As New Proj4Projection
    Dim sUri As String = String.Empty

    'sUri = "http://geoportal.trinity.duke.edu/geoserver/gwc/service/wmts/"
    'sUri = "https://basemap.nationalmap.gov/arcgis/rest/services/USGSImageryOnly/MapServer/WMTS/"
    'sUri = "https://basemap.nationalmap.gov/arcgis/rest/services/USGSImageryOnly/MapServer/WMTS/"
    'sUri = "https://basemap.nationalmap.gov/arcgis/rest/services/USGSShadedReliefOnly/MapServer/WMTS/"


    sUri = "https://basemap.nationalmap.gov/arcgis/rest/services/USGSTopo/MapServer/WMTS/"
    moWmtsLayer.ServerUris.Add(New Uri(sUri))
    moWmtsLayer.DrawingExceptionMode = ThinkGeo.MapSuite.Drawing.DrawingExceptionMode.DrawException
    moWmtsLayer.WmtsSeverEncodingType = ThinkGeo.MapSuite.Layers.WmtsSeverEncodingType.Restful
    moWmtsLayer.Open()

    'XmlToDataset(moWmtsLayer)
    'Dim oLayerForm As New frmChooseLayer With {.LayerMatrixTable = moTableMatrixLink}
    'Dim oItem As UltraListViewItem


    'With oLayerForm
    '  '***** Populate the listview
    '  For Each oRow As DataRow In moLayer.Rows
    '    If mLayerHasStyle(oRow("Layer_ID")) Then
    '      oItem = New UltraListViewItem(oRow("Layer_Id"))
    '      oItem.Value = oRow("Title")
    '      oItem.Tag = oRow("Identifier")
    '      .lvwLayers.Items.Add(oItem)
    '    End If
    '  Next oRow




    '  Dim eResult = .ShowDialog()
    '  If eResult = DialogResult.Cancel Then
    '    Me.Close()
    '    Return
    '  End If
    'End With

    'Dim oProps = oLayerForm.LayerProps
    Dim oLayerProps As New WmtsLayerProps
    With oLayerProps
      .LayerName = "USGSTopo"
      .LayerId = "USGSTopo"
      .Srid = "GoogleMapsCompatible"
      .Identifier = "USGSTopo"
    End With

    mAddWmtsLayer(oLayerProps)
    WinformsMap1.Refresh()

    Dim sPath As String = "C:\Temp\TestSerializeWmtsLayer"
    If Not IO.Directory.Exists(sPath) Then
      IO.Directory.CreateDirectory(sPath)
    End If

    Dim oSerializer As New GeoSerializer
    Try
      moWmtsLayer.Close()
      oSerializer.Serialize(moWmtsLayer, sPath)
    Catch ex As Exception
      MessageBox.Show(ex.Message, "Yikes!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
    End Try


  End Sub
  Private Sub mAddWmtsLayer(oLayerProps As WmtsLayerProps)
    With moWmtsLayer
      .ActiveLayerName = oLayerProps.Identifier
      .ActiveStyleName = "default"
      .TileMatrixSetName = oLayerProps.Srid
    End With

    Dim oOverlay As New LayerOverlay
    WinformsMap1.MapUnit = ThinkGeo.MapSuite.GeographyUnit.Meter


    oOverlay.Layers.Add(moWmtsLayer)

    WinformsMap1.Overlays.Add(oOverlay)
    Dim oRect As New RectangleShape(8466755.57232525, 4918863.47372144, 14332212.7026385, 1699955.39053035)
    WinformsMap1.CurrentExtent = oRect

  End Sub
  Private Function mGetProjectionToGoogle() As Proj4Projection
    Dim oProj As New Proj4Projection
    oProj.InternalProjectionParametersString = Proj4Projection.GetWgs84ParametersString
    oProj.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString
    oProj.Open()
    Return oProj
  End Function
  Private Sub moWmtsLayer_SendingWebRequest(sender As Object, e As SendingWebRequestEventArgs) Handles moWmtsLayer.SendingWebRequest
    Debug.WriteLine(e.WebRequest.RequestUri.AbsoluteUri)
  End Sub
  Private Sub XmlToDataset(oWmtsLayer As WmtsLayer)
    Dim xmlFile As XmlReader
    Dim sCapabilities As String = oWmtsLayer.GetServerCapabilitiesXml()

    Dim sFileName As String = sDataPath + "Capabilities.xml"
    IO.File.WriteAllText(sFileName, sCapabilities)
    xmlFile = XmlReader.Create(sFileName, New XmlReaderSettings())

    Dim ds As DataSet = New DataSet()
    Dim oRowData As New Text.StringBuilder
    ds.ReadXml(xmlFile)


    For Each oTable As DataTable In ds.Tables
      Debug.WriteLine(oTable.TableName)
      Select Case oTable.TableName
        Case "Layer"
          moLayer = oTable
        Case "Style"
          moLayerStyle = oTable
          'mDisplayTableContents(oTable)
        Case "TileMatrixSetLink"
          moTableMatrixLink = oTable
          'mDisplayTableContents(oTable)
        Case "WGS84BoundingBox"
          moWGS84BoundingBox = oTable
          mDisplayTableContents(oTable)
      End Select
    Next oTable
  End Sub
  Private Sub mDisplayTableContents(oTable As DataTable)
    Dim oRowData As New Text.StringBuilder
    For Each oRow As DataRow In oTable.Rows
      For Each oCol As DataColumn In oTable.Columns
        oRowData.Append(oCol.ColumnName + ": " + oRow.Item(oCol.ColumnName).ToString + " ")
      Next oCol
      Debug.WriteLine(oRowData.ToString)
      oRowData.Clear()
    Next oRow
  End Sub
  Private Function mGetStyleThing(sLayerId As String) As String()
    Dim oRow = moLayerStyle.Select("Layer_Id=" + sLayerId)
    Dim asSrid(oRow.Length) As String
    For i = 0 To oRow.Length - 1
      asSrid(i) = oRow(i)("Identifier").ToString
    Next i
    Return asSrid
  End Function
  Private Function mLayerHasStyle(sLayerId As String) As Boolean
    Dim asStyles = mGetStyleThing(sLayerId)
    For Each s In asStyles
      If Not s = String.Empty Then
        Return True
      End If
    Next
    Return False
  End Function
  Private Function mGetWgs84ExtentForLayer(sLayerId As String) As RectangleShape
    Dim oLowerLeftPoint As PointShape
    Dim oUpperRightPoint As PointShape
    Dim oRect As RectangleShape
    Dim dX As Double
    Dim dY As Double
    Dim sCorner() As String

    Dim oRow = moWGS84BoundingBox.Select("Layer_Id=" + "0")
    sCorner = oRow(0)("LowerCorner").ToString.Split(" ")
    dX = Double.Parse(sCorner(0))
    dY = Double.Parse(sCorner(1))
    oLowerLeftPoint = New PointShape(dX, dY)

    sCorner = oRow(0)("UpperCorner").ToString.Split(" ")
    dX = Double.Parse(sCorner(0))
    dY = Double.Parse(sCorner(1))
    oUpperRightPoint = New PointShape(dX, dY)

    oRect = New RectangleShape(oLowerLeftPoint.X, oUpperRightPoint.Y, oUpperRightPoint.X, oLowerLeftPoint.Y)
    Return oRect
  End Function


  Private Sub WinformsMap1_CurrentExtentChanged(sender As Object, e As CurrentExtentChangedWinformsMapEventArgs) Handles WinformsMap1.CurrentExtentChanged
    Debug.WriteLine(e.CurrentExtent.ToString)
  End Sub
End Class
